@@ -175,6 +175,24 @@ class TestWaitConditions(unittest.TestCase):
     def test_legacy_explicit_wait_type(self):
         Retrying(wait="exponential_sleep")
 
+    def test_wait_func_result(self):
+        captures = []
+
+        def wait_capture(attempt, delay, last_result=None):
+            captures.append(last_result)
+            return 1
+
+        def dying():
+            raise Exception("Broken")
+
+        r_attempts = 10
+        r = Retrying(wait=wait_capture, sleep=lambda secs: None,
+                     stop=tenacity.stop_after_attempt(r_attempts),
+                     reraise=True)
+        self.assertRaises(Exception, r.call, dying)
+        self.assertEqual(r_attempts - 1, len(captures))
+        self.assertTrue(all([r.failed for r in captures]))
+
     def test_wait_func(self):
         r = Retrying(wait=lambda attempt, delay: attempt * delay)
         self.assertEqual(r.wait(1, 5), 5)
@@ -702,6 +720,27 @@ class TestBeforeAfterAttempts(unittest.TestCase):
 
         self.assertTrue(TestBeforeAfterAttempts._attempt_number is 2)
 
+    def test_before_sleep(self):
+        TestBeforeAfterAttempts._attempt_number = 0
+
+        def _before_sleep(retry_obj, sleep, last_result):
+            self.assertGreater(sleep, 0)
+            TestBeforeAfterAttempts._attempt_number = \
+                retry_obj.statistics['attempt_number']
+
+        @retry(wait=tenacity.wait_fixed(0.1),
+               stop=tenacity.stop_after_attempt(3),
+               before_sleep=_before_sleep)
+        def _test_before_sleep():
+            if TestBeforeAfterAttempts._attempt_number < 2:
+                raise Exception("testing before_sleep_attempts handler")
+            else:
+                pass
+
+        _test_before_sleep()
+
+        self.assertTrue(TestBeforeAfterAttempts._attempt_number is 2)
+
 
 class TestReraiseExceptions(unittest.TestCase):
 
@@ -792,5 +831,31 @@ class TestStatistics(unittest.TestCase):
         self.assertEqual(2, _foobar.retry.statistics['attempt_number'])
 
 
+class TestRetryErrorCallback(unittest.TestCase):
+
+    def setUp(self):
+        self._attempt_number = 0
+        self._callback_called = False
+
+    def _callback(self, fut):
+        self._callback_called = True
+        return fut
+
+    def test_retry_error_callback(self):
+        num_attempts = 3
+
+        @retry(stop=tenacity.stop_after_attempt(num_attempts),
+               retry_error_callback=self._callback)
+        def _foobar():
+            self._attempt_number += 1
+            raise Exception("This exception should not be raised")
+
+        result = _foobar()
+
+        self.assertTrue(self._callback_called)
+        self.assertEqual(num_attempts, self._attempt_number)
+        self.assertIsInstance(result, tenacity.Future)
+
+
 if __name__ == '__main__':
     unittest.main()
